// Autor    : kk
// EMail    : leluno@gmx.de

//              PA4  -> DCMI_HSYNC  = Camera HSYNC (HREF)
//              PA6  -> DCMI_PCLK   = Camera PCLK (PIXCLK)
//              //PB6  -> DCMI_D5     = Camera D5
//              PB7  -> DCMI_VSYNC  = Camera VSYNC
//              PB8  -> I2C1-SCL    = Camera I2C-SCL
//              PB9  -> I2C1-SDA    = Camera I2C-SDA
//              PC6  -> DCMI_D0     = Camera D0
//              PC7  -> DCMI_D1     = Camera D1
//             (PC8) -> DCMI_D2     = Camera D2
//             (PC9) -> DCMI_D3     = Camera D3
//              PD3  -> DCMI_D5     = Camera D5
//              PE4  -> DCMI_D4     = Camera D4
//              PE5  -> DCMI_D6     = Camera D6
//              PE6  -> DCMI_D7     = Camera D7
//				pc14 -> PWDN
//
//              DMA :  DMA2_STREAM1_CHANNEL1

#define cam_ov7670 0
#define cam_ov3640 1
#define cam_mtd112 2//not yet tested
#define cam_chip 1

volatile uint8_t registerValue;

#define DELAY_MS 0
#define ARRAY_END 0xff


typedef struct CamReg_t{
    uint16_t addr;
    uint8_t data;
} CamReg;




#if cam_chip==cam_ov7670
#define cam_adr 0x42
#define reg_datei ov7670_reg
#define vsync_signal DCMI_VSPolarity_High
#define hline_signal DCMI_HSPolarity_Low
#define pclk_polarity DCMI_PCKPolarity_Rising
#define sccb_bus_wr sccb_wr_8_8
#define sccb_bus_rd sccb_rd_8_8
//#define cam_format  cam_formatVGA
u32 h_px=640, v_px=480;
#define cam_format_xQVGA  2
#define regDateiFile  "ov7670_regs.h"
#endif

#if cam_chip==cam_ov3640
#define cam_adr 0x78
#define vsync_signal DCMI_VSPolarity_Low
#define hline_signal DCMI_HSPolarity_Low
#define pclk_polarity DCMI_PCKPolarity_Rising
//#define pclk_polarity DCMI_PCKPolarity_Falling
#define sccb_bus_wr sccb_wr_16_8
#define sccb_bus_rd sccb_rd_16_8
#define cam_format_xQVGA  3
u32 h_px=1024, v_px=768;
#define reg_datei ov3640_xga
#define regDateiFile "ov3640_regxxx.h"
#endif

#if cam_chip==cam_mtd112//not tested
#define cam_adr 0xba
#define vsync_signal DCMI_VSPolarity_Low
#define hline_signal DCMI_HSPolarity_Low
#define pclk_polarity DCMI_PCKPolarity_Rising
#define sccb_bus_wr sccb_wr_16_16
#define sccb_bus_rd sccb_rd_16_16
//#define cam_format  cam_formatQVGA
#define cam_format_xQVGA  2
#define reg_datei mtd112_reg
#define regDateiFile "mt9d112_regs.h"
#endif

/*#if cam_format==cam_formatQVGA
#define H_PX 320
#define V_PX 240
#endif
#if cam_format==cam_formatVGA*/
#define H_PX 320*cam_format_xQVGA
#define V_PX 240*cam_format_xQVGA
/*#endif
#if cam_format==cam_formatUXGA
//#define H_PX 1600
//#define V_PX 1200
#define H_PX 320*3
#define V_PX 240*3
#endif
*/
#include regDateiFile

#define DCMI_DR_REGISTER 	DCMI_BASE|0x28
#define SDRAM_START_ADRx      ((uint32_t)0xD0000000)
#define cam_bufferStart				SDRAM_START_ADRx
//#define H_PX 640
//#define V_PX 480
#define pwdn_high		pinSET(C,14)
#define pwdn_low		pinCLR(C,14)

volatile uint16_t *pixel_cursorDB=cam_bufferStart+H_PX<<3;//>*10;
volatile uint32_t zl_Hline;

void DMA2_Stream1_IRQHandler(void){
  if(DMA_GetITStatus(DMA2_Stream1, DMA_IT_TCIF1)){//TransferComplete
    DMA_ClearITPendingBit(DMA2_Stream1, DMA_IT_TCIF1);
	  zl_Hline+=8;
	  pixel_cursorDB+=h_px<<3;//*8
	  if((zl_Hline)%16){//line0 ready - start line1
		  DMA2_Stream1->M0AR=0xD0000000+4*h_px*zl_Hline;
	  }else{//line1 ready - start line0
		  DMA2_Stream1->M1AR=0xD0000000+4*h_px*zl_Hline;
	  }
  }
}

void cam_InitXCLK(){
	  GPIO_InitTypeDef GPIO_InitStructure;
//	  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA,ENABLE);
	  GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_MCO);
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
//	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
//	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOA, &GPIO_InitStructure);

	  //HSE 8mhz HSI 16mhz PLLC 168mhz
//	  RCC_MCO1Config(RCC_MCO1Source_HSE,RCC_MCO1Div_1);
//	  RCC_MCO1Config(RCC_MCO1Source_HSI,RCC_MCO1Div_1);
	  RCC_MCO1Config(RCC_MCO1Source_PLLCLK,RCC_MCO1Div_5);

//Pwdn
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_Init(GPIOC, &GPIO_InitStructure);
	//GPIOG->OSPEEDR=	GPIO_Speed_50MHz<<(2*pin);
	}
void cam_InitIO(void)
{
	  GPIO_InitTypeDef GPIO_InitStructure;

	  GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_DCMI); // PA4=DCMI_HSYNC -> HSNYC
	  GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_DCMI); // PA6=DCMI_PCLK  -> PCLK

	  // Structur fuer Port-A
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6;// | GPIO_Pin_8;

	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  // Config von Port-A
	  GPIO_Init(GPIOA, &GPIO_InitStructure);

	  //-----------------------------------------
	  // Alle Pins von Port-B initialisieren
	  //-----------------------------------------
	  //GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_DCMI); // PB6=DCMI_D5    -> D5
	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_DCMI); // PB7=DCMI_VSYNC -> VSYNC

	  // Structur fuer Port-B
	  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_7;

	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  // Config von Port-B
	  GPIO_Init(GPIOB, &GPIO_InitStructure);

	  //-----------------------------------------
	   // Alle Pins von Port-C initialisieren
	   //-----------------------------------------
	   GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_DCMI); // PC6=DCMI_D0    -> D0
	   GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_DCMI); // PC7=DCMI_D1    -> D1
	   GPIO_PinAFConfig(GPIOC, GPIO_PinSource8, GPIO_AF_DCMI); // PC8=DCMI_D2    -> D2
	   GPIO_PinAFConfig(GPIOC, GPIO_PinSource9, GPIO_AF_DCMI); // PC9=DCMI_D3    -> D3

	   // Structur fuer Port-C
	   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;

	   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	   GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	   GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	   // Config von Port-C
	   GPIO_Init(GPIOC, &GPIO_InitStructure);

	   //-----------------------------------------
	    // Alle Pins von Port-d initialisieren
	    //-----------------------------------------
	    GPIO_PinAFConfig(GPIOD, GPIO_PinSource3, GPIO_AF_DCMI); // PC6=DCMI_D0    -> D0

	    // Structur fuer Port-d
	    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;

	    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	    // Config von Port-d
	    GPIO_Init(GPIOD, &GPIO_InitStructure);

	  //-----------------------------------------
	  // Alle Pins von Port-E initialisieren
	  //-----------------------------------------
//	  GPIO_PinAFConfig(GPIOE, GPIO_PinSource0, GPIO_AF_DCMI); // PE0=DCMI_D2    -> D4
//	  GPIO_PinAFConfig(GPIOE, GPIO_PinSource1, GPIO_AF_DCMI); // PE1=DCMI_D3    -> D4
	  GPIO_PinAFConfig(GPIOE, GPIO_PinSource4, GPIO_AF_DCMI); // PE4=DCMI_D4    -> D4
	  GPIO_PinAFConfig(GPIOE, GPIO_PinSource5, GPIO_AF_DCMI); // PE5=DCMI_D6    -> D6
	  GPIO_PinAFConfig(GPIOE, GPIO_PinSource6, GPIO_AF_DCMI); // PE6=DCMI_D7    -> D7

	  // Structur fuer Port-E
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6;//|GPIO_Pin_0 |GPIO_Pin_1 ;

	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  // Config von Port-E
	  GPIO_Init(GPIOE, &GPIO_InitStructure);


	  //HSE 8mhz HSI 16mhz PLLC 168mhz
	//  RCC_MCO1Config(RCC_MCO1Source_HSI,RCC_MCO1Div_1);
	  RCC_MCO1Config(RCC_MCO1Source_PLLCLK,RCC_MCO1Div_5);
	  pinDIRout(B,9);
	}

void mco_xclock_on (void){
  RCC->CFGR |= RCC_MCO1Source_PLLCLK;
}

void mco_xclock_off (void){
  RCC->CFGR &=~ RCC_MCO1Source_PLLCLK;
}

void powerDnSeq(void){
mco_xclock_off();
pwdn_low;
}

void cam_powerUpSeq(void){
powerDnSeq();
_delay_ms(300);
pwdn_high;
_delay_ms(11);
mco_xclock_on();
_delay_ms(11);
pwdn_low;
_delay_ms(20);
}



void cam_init_sccb_io(void) {
  GPIO_InitTypeDef GPIO_InitStructure;
  // Clock Enable
//RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  // Config PG13 als Digital-Ausgang
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
}




void cam_InitDCMI(void){
	  DCMI_InitTypeDef DCMI_InitStructure;

	  // Clock enable
	  RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);

	  // DCMI init
	  DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_SnapShot;
	//  DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_Continuous;
	  DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;

	  DCMI_InitStructure.DCMI_PCKPolarity = pclk_polarity;
//	  DCMI_InitStructure.DCMI_PCKPolarity = DCMI_PCKPolarity_Rising;//of7670
//	  DCMI_InitStructure.DCMI_PCKPolarity = DCMI_PCKPolarity_Falling;

	  DCMI_InitStructure.DCMI_VSPolarity = vsync_signal;
//	  DCMI_InitStructure.DCMI_VSPolarity = DCMI_VSPolarity_High;//of7670

	  DCMI_InitStructure.DCMI_HSPolarity = hline_signal;
	  //DCMI_InitStructure.DCMI_HSPolarity = DCMI_HSPolarity_Low;//of7670

	  DCMI_InitStructure.DCMI_CaptureRate = DCMI_CaptureRate_1of4_Frame;
	  DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b;
	  DCMI_Init(&DCMI_InitStructure);
}

//--------------------------------------------------------------
void cam_InitDMA(void){
	  DMA_InitTypeDef  DMA_InitStructure;

	  // Clock enable
	  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);

	  // DMA deinit
	  DMA_Cmd(DMA2_Stream1, DISABLE);
	  DMA_DeInit(DMA2_Stream1);

	  // DMA init

	  DMA_InitStructure.DMA_Channel = DMA_Channel_1;
	  DMA_InitStructure.DMA_PeripheralBaseAddr =&DCMI->DR;// OV9655_DCMI_REG_DR_ADDRESS;//
	  DMA_InitStructure.DMA_Memory0BaseAddr = 0xD0000000;//LCD_RAM_ADR;
	  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
	  DMA_InitStructure.DMA_BufferSize = h_px*8;
	  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	//  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
	  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;

	  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
	  //DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;

	 // DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	  //DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	   DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word;

	  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	  DMA_InitStructure.DMA_Priority = DMA_Priority_High;

	//  DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
	  DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Enable;
	  DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
	  DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
	  DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
	//  DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_INC4;
	  DMA_Init(DMA2_Stream1, &DMA_InitStructure);

	  DMA_DoubleBufferModeCmd(DMA2_Stream1, ENABLE);
	//  DMA_MemoryTargetConfig(DMA2_Stream1,pixel_cursorDB+100000, DMA_Memory_1);
	  DMA2_Stream1->M1AR=0xD0000000+h_px*8*4;
	  DMA_ITConfig(DMA2_Stream1,DMA_IT_TC,ENABLE);
	}


void cam_CaptureStart(void){
	  // Cursor auf Start setzen
	//  UB_LCD_SetCursor2Draw(LCD_WINDOW.xstart,LCD_WINDOW.ystart);
	  // DMA enable
//	cam_InitDCMI();
	  zl_Hline=0;
	  DMA2_Stream1->M0AR=0xD0000000;
	  DMA2_Stream1->M1AR=0xD0000000+H_PX*8*4;
	cam_InitDMA();
	 DMA_Cmd(DMA2_Stream1, ENABLE);
	  // DCMI enable
	  DCMI_Cmd(ENABLE);
	  // Capture enable
	  DCMI_CaptureCmd(ENABLE);
	}



//--------------------------------------------------------------
// stoppen vom Capture-Mode
// und stopen vom LCD zugriff
//--------------------------------------------------------------
void camx_CaptureStop(void)
{
  // Capture disable
  DCMI_CaptureCmd(DISABLE);
  // DCMI disable
  DCMI_Cmd(DISABLE);
  // DMA disable
  DMA_Cmd(DMA2_Stream1, DISABLE);
}

void DMA2Stream1_InitNVIC(void){
	  NVIC_InitTypeDef NVIC_InitStructure;
	  DMA_ITConfig(DMA2_Stream1, DMA_IT_TC, ENABLE);
		  NVIC_InitStructure.NVIC_IRQChannel = DMA2_Stream1_IRQn;
		  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
		  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
		  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	  NVIC_Init(&NVIC_InitStructure);

}

void cam_init(void){
	  // init vom Camera-Modul

}
