/*
================================================================================
*             Texas Instruments OMAP(TM) Platform Software
* (c) Copyright Texas Instruments, Incorporated. All Rights Reserved.
*
* Use of this software is controlled by the terms and conditions found
* in the license agreement under which this software has been supplied.
*
================================================================================
*/
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef __OV3640_H_INCLUDED__
#define __OV3640_H_INCLUDED__

#define I2C2_MEMBASE_ADDR               0x48072000
#define OV3640_I2C_DEVICE_ADDR          0x3C  // Address in 3430 SDPES2.0
#define OV3640_REG_PROGRAM_REGION_START 0x8000
#define OV3640_REG_PIDH                 0x300A
#define OV3640_REG_PIDL                 0x300B
#define OV3640_REG_SCCB_ID              0x300C
#define OV3640_REG_SC_SYN_CTRL0         0x3104
#define OV3640_REG_SC_SYN_CTRL1         0x3105

#define OV3640_MASK_SC_SYN_CTRL0_MC_TOP_RST 0x02
#define OV3640_MASK_SC_SYN_CTRL1_MC_TOP_CLK 0x02

#define OV3640_ADDRSIZE   16
#define OV3640_PID        0x36      // camera id
#define OV3640_DEFAULT_POSTPROC 0

#define OV3640_EXCLK_MIN  24000000 // min exclk
#define OV3640_EXCLK_MAX  24000000 // max exclk

// override for default MDD camera properties
#define OV3640_BRIGHTNESS_MIN   (-3)
#define OV3640_BRIGHTNESS_MAX   (3)
#define OV3640_BRIGHTNESS_STEP  (1)
#define OV3640_BRIGHTNESS_DEF   (0)

// override for default MDD camera properties
#define OV3640_CONTRAST_MIN   (0)
#define OV3640_CONTRAST_MAX   (4)
#define OV3640_CONTRAST_STEP  (1)
#define OV3640_CONTRAST_DEF   (4)  // noise reduction is the default


#define I2C_DEVICE_NAME          L"I2C1:"
#define I2C_CAMERA_ADDRSIZE      OV3640_ADDRSIZE
#define I2C_CAMERA_ADDRESS       OV3640_I2C_DEVICE_ADDR

#define STATE_CHANGE_TICK_COUNT 5000  // 500 ms to allow for camera to change state.

// Max number of items allowed in I2C_MULTI_OPERATION structure.
// choosing 30 as somewhat arbitrary number, know it has a max UINT8 buffer of 1030, want to stay well under
#define MAX_I2C_MULTI_OP_STRUCTS    30


//
// OV3640 subaddresses
//

#define OV_ID_REG     0x00


typedef enum
{
    I2C_ERROR,
    I2C_OK
} I2CERROR;

typedef enum
{
    CAM_RESET,      // off or standby
    CAM_STOP,       // powerup but inactive
    CAM_PAUSE,      // ready to run
    CAM_RUN         // capture or preview mode
} CAMSTATE;



typedef enum
{
    YUV422_YUY2,
    YUV422_UYVY,
    IJPG,
    BGR565,
    RAW10,
    RAW08,
    N_MAX_FORMAT
} CAMFORMAT;

typedef enum
{
    // matching sensor states except unknown - add more if necessary.
    MODE_PREVIEW=3,
    MODE_CAPTURE=7,
    MODE_STANDBY=9,
    MODE_UNKNOWN=10
} CAMMODE;

typedef enum
{
    QQVGA=0,
    QCIF,
    QVGA,
    VGA,
    XGA,
    N_MAX_RES,
    CAMRES_INVALID=0xffffffff
} CAMRES;


/* Camera Frame Rate enumerations */
typedef enum
{
  FPS_15,
  FPS_30,
  MAX_FRAMERATE
}CAMFRAMERATE;

typedef enum
{
    BIT8,
	BIT16,
	BIT32,
//	DELAY_MS,//kk
//	ARRAY_END
} DATAWIDTH;

//
// Global structures and variable definitions
//

#define LONG u16
#define BYTE u8
#define DWORD u32
#define WORD u32
struct FormatTable_t
{
    LONG    width;
    LONG    height;
    DWORD   bitcount;
    DWORD   format;
    DWORD   postprocinfo;
    DWORD   skipframes;
    DWORD   fps;
    DWORD   max_exposure;
    DWORD   avgtimeperframe; // 100ns units
};

typedef struct CamRegValue_t
{
    BYTE SubAddr;
    BYTE Value;
} CamRegValue;

/*typedef struct CamReg_t{
    uint16_t addr;
    uint8_t data;
} CamReg;*/

#pragma pack(1)
//
// OV3640 Sensor commands list
//

static CamReg ov3640_xga[] = {
//		static const SensorInit_CommandsList_XGABase[130][2] = {
    { 0x3012, 0x80},  /*Software reset the sensor */
    { DELAY_MS,  0x05},    /* delay 5 milliseconds */

    /* OV3640_Parallel_XGA_YUV 30 fps - Default parallel code
     * 24 MHz input clock
     * Common Configuration for CSI2 and Parallel mode data
     */
    { 0x304d, 0x45},  /* ND */
    { 0x3087, 0x16},  /* TMC11 -> bit 2 is not used, 1:0 is pixel order for color bar use! */
    { 0x30aa, 0x45},  /*  not in data sheet */
    { 0x30b0, 0xff},  /*  IO_Control CY7:0 - output */
    { 0x30b1, 0xff},  /* IOC_CONTROL1 ->all output */
    { 0x30b2, 0x10},  /* IO_CONTROL2 ->but only bits 0 and 1 are defined in datasheet (page 19) */
    { 0x300e, 0x32},  /* PLL */
    { 0x300f, 0x21},  /* PLL */
    { 0x3010, 0x21},  /* PLL */
    { 0x3011, 0x01},  /* PLL Clock divider kk*/
    { 0x304c, 0x82},  /* Not Defined */
    { 0x3018, 0x38},  /* WPT/HIST */
    { 0x3019, 0x30},  /* BPT/HIST */
    { 0x301a, 0x61},  /* VPT */
    { 0x307d, 0x00},  /* TMC7-- */
    { 0x3087, 0x02},  /* TMC11 */
    { 0x3082, 0x20},  /*  Not Defined: RSRDVD */
    { 0x3015, 0x12},  /* AUTO_3: agc_ceil=8x, 1 dummy frame */
    { 0x3014, 0x04},  /* AUTO_2 */
    { 0x3013, 0xf7},  /* AUTO_1 */
    { 0x303c, 0x08},  /* RSVD/AHW */
    { 0x303d, 0x18},  /* AHW */
    { 0x303e, 0x06},  /* RSVD/AVH */
    { 0x303f, 0x0c},  /* AVH */
    { 0x3030, 0x62},  /* AVG Section Weighting (Start) */
    { 0x3031, 0x26},
    { 0x3032, 0xe6},
    { 0x3033, 0x6e},
    { 0x3034, 0xea},
    { 0x3035, 0xae},
    { 0x3036, 0xa6},
    { 0x3037, 0x6a},  /* AVG Section Weighting (END) */
    { 0x3104, 0x02},  /* SC_SYN_CTRL0 */
    { 0x3105, 0xfd},  /* SC_SYN_CTRL1 */
    { 0x3106, 0x00},  /* SC_SYN_CTRL2 */
    { 0x3107, 0xff},  /* SC_SYN_CTRL3 */
    { 0x3300, 0x12},  /* DSP_CTRL0 */
    { 0x3301, 0xde},  /* DSP_CTRL1 */
    { 0x3302, 0xef},  /* DSP_CTRL2 */
    { 0x3316, 0xff},  /* Not Defined */
    { 0x3317, 0x00},  /* Not Defined */
    { 0x3308, 0xa5},  /* AWB_CTRL3 */
    { 0x3312, 0x26},  /* Not defined */
    { 0x3314, 0x42},  /* Not defined */
    { 0x3313, 0x2b},  /* Not Defined */
    { 0x3315, 0x42},  /* Not Defined */
    { 0x3310, 0xd0},  /* Not Defined */
    { 0x3311, 0xbd},  /* Not Defined */
    { 0x330c, 0x18},  /* Not Defined */
    { 0x330d, 0x18},  /* Not Defined */
    { 0x330e, 0x56},  /* Not Defined */
    { 0x330f, 0x5c},  /* Not Defined */
    { 0x330b, 0x1c},  /* Not Defined */
    { 0x3306, 0x5c},  /* Not Defined */
    { 0x3307, 0x11},  /* Reserved */
    { 0x336a, 0x52},  /* R_A1 */
    { 0x3370, 0x46},  /* G_A1 */
    { 0x3376, 0x38},  /* B_A1 */
    { 0x30b8, 0x20},  /* Not Defined */
    { 0x30b9, 0x17},  /* Not Defined */
    { 0x30ba, 0x04},  /* DSPC2 */
    { 0x30bb, 0x08},  /* DSPC3 */
    { 0x3507, 0x06},  /* Not Defined */
    { 0x350a, 0x4f},  /* Not Defined */
    { 0x3100, 0x02},  /* SC_CTRL0 */
    { 0x3301, 0xde},  /* DSP_CTRL1 */
    { 0x3304, 0},		//kk 0xfc},  /* DSP_CTRL_4 */
    { 0x3400, 1},		//kk 0x00},  /* FMT_MUX_CTRL0 */
    { 0x3404, 0x11}, 	//kk 0x00},  /* FMT_CTRL00: RGB565 - 10,11 or 30, YUV4.2.2 0x0 */
    { 0x3012, 0x10},  /* SYS=XGA */
    { 0x3023, 0x06},  /* VS */
    { 0x3026, 0x03},  /* VH */
    { 0x3027, 0x04},  /* VH */
    { 0x302a, 0x03},  /* VTS */
    { 0x302b, 0x10},  /* VTS */
    { 0x3075, 0x24},  /* VSYCOPT */
    { 0x300d, 0x01},  /* PCLK */
    { 0x30d7, 0x90},  /* Not Defined */
    { 0x3069, 0x04},  /* BLC9 */
    { 0x303e, 0x00},  /* RSVD/AVH */
    { 0x303f, 0xc0},  /* AVH */
    { 0x335f, 0x34},  /* Not Defined */
    { 0x3360, 0x0c},  /* Not Defined */
    { 0x3361, 0x04},  /* Not Defined */
    { 0x3362, 0x34},  /* Not Defined */
    { 0x3363, 0x08},  /* HSIZE_OUT_L */
    { 0x3364, 0x04},  /* VSIZE_OUT_L */
    { 0x3403, 0x42},  /* ISP_PAD_CTR2 */
    { 0x3088, 0x04},  /* ISP_XOUT */
    { 0x3089, 0x00},  /* ISP_XOUT */
    { 0x308a, 0x03},  /* ISP_YOUT */
    { 0x308b, 0x00},  /* ISP_YOUT */
    { 0x308d, 0x04},  /* TMC13 - reset block sleep enable (no mipi disable??) */
    { 0x3086, 0x03},  /* TMC10 */
    { 0x3086, 0x00},  /* TMC10 */
    { 0x3600, 0xc4},  /* OUT_CTRL00 */
/* End of Normal Parallel Code */
#if 0
    { 0x3602, 0x22},  /* NM OUT_CONTROL2 SOL/EOL off */
    { 0x360c, 0x12},  /* virtual channel 0 rsvd mipi_ctrl0c */	 //Bits 7:6 channel ID
    { 0x361e, 0x00},  //NM OUT_CONTROL1E h_sel?
    { 0x361f, 0x11},  /* mipi_ctrl1f pclk_period (ns) */
    { 0x3623, 0x01},  /* mipi_ctrl23 increase hs_prepare */
    { 0x3633, 0x01},  /* mipi_ctrl33 increase hs_prepare */
    { 0x3626, 0x00},  /* mipi_ctrl26 increase clk_prepare */
    { 0x3627, 0x08},  /* mipi_ctrl27 increase clk_prepare */
    { 0x3629, 0x30},  /* mipi_ctrl29 increase clk_prepare */
    { 0x3603, 0x4D},  /* MIPI_CTRL3 */
    { 0x364c, 0x04},  /* MIPI_CTRL4C */
    { 0x3010, 0x21},  /* PLL_3 lower mipi spd, 81 */
    { 0x3507, 0x06},
    { 0x350a, 0x4f},
    { 0x3600, 0xc4},
    { 0x3602, 0x20},  /* SOL off */
    { 0x3104, 0x02},
    { 0x3602, 0x22},  /* SOL/EOL off */
    { 0x309e, 0x00},
    { 0x3602, 0x22},  /* SOL/EOL off */

#endif
    {ARRAY_END, 0xFF}
};

static CamReg SensorInit_CommandsList_QXGABase[] = {
    {  0x3012, 0x80},
    {  0x304d, 0x45},
    {  0x3087, 0x16},
    {  0x30aa, 0x45},
    {  0x30b0, 0xff},
    {  0x30b1, 0xff},
    {  0x30b2, 0x10},
    {  0x30d7, 0x10},

    {  0x309e, 0x00},
    {  0x3602, 0x26},
    {  0x3603, 0x4D},
    {  0x364c, 0x04},
    {  0x360c, 0x12},
    {  0x361e, 0x00},
    {  0x361f, 0x11},
    {  0x3633, 0x32},
    {  0x3629, 0x3c},
    {  0x300e, 0x32},
    {  0x300f, 0x21},
    {  0x3010, 0x21},
    {  0x3011, 0x00},
    {  0x304c, 0x81},
    {  0x3018, 0x38},
    {  0x3019, 0x30},
    {  0x301a, 0x61},
    {  0x307d, 0x00},
    {  0x3087, 0x02},
    {  0x3082, 0x20},

    {  0x303c, 0x08},
    {  0x303d, 0x18},
    {  0x303e, 0x06},
    {  0x303F, 0x0c},
    {  0x3030, 0x62},
    {  0x3031, 0x26},
    {  0x3032, 0xe6},
    {  0x3033, 0x6e},
    {  0x3034, 0xea},
    {  0x3035, 0xae},
    {  0x3036, 0xa6},
    {  0x3037, 0x6a},

    {  0x3015, 0x12},
    {  0x3014, 0x04},
    {  0x3013, 0xf7},

    {  0x3104, 0x02},
    {  0x3105, 0xfd},
    {  0x3106, 0x00},
    {  0x3107, 0xff},
    {  0x3308, 0xa5},
    {  0x3316, 0xff},
    {  0x3317, 0x00},
    {  0x3087, 0x02},
    {  0x3082, 0x20},
    {  0x3300, 0x13},
    {  0x3301, 0xd6},
    {  0x3302, 0xef},

    {  0x30b8, 0x20},
    {  0x30b9, 0x17},
    {  0x30ba, 0x04},
    {  0x30bb, 0x08},

    //Format
    {  0x3100, 0x02},
    {  0x3304, 0xfc},
    {  0x3400, 0x00},
    {  0x3404, 0x00},

    //Size
    {  0x3020, 0x01},
    {  0x3021, 0x1d},
    {  0x3022, 0x00},
    {  0x3023, 0x0a},
    {  0x3024, 0x08},
    {  0x3025, 0x18},
    {  0x3026, 0x06},
    {  0x3027, 0x0c},
    {  0x335f, 0x68},
    {  0x3360, 0x18},
    {  0x3361, 0x0c},
    {  0x3362, 0x68},
    {  0x3363, 0x08},
    {  0x3364, 0x04},
    {  0x3403, 0x42},
    {  0x3088, 0x08},
    {  0x3089, 0x00},
    {  0x308a, 0x06},
    {  0x308b, 0x00},

    {  0x3507, 0x06},
    {  0x350a, 0x4f},
    {  0x3600, 0xc4},
    {  0x3011, 0x01},

    {0xFF, 0xFF,   0x00}
};

static CamReg SensorInit_RAWMode[] =
{
         //Manual Exposure
    {0x3000, 0x00},
    {0x3001, 0x80},
    {0x3002, 0x04},
    {0x3003, 0x00},
    {0x3004, 0x00},
    {0x3005, 0x40}, //RED
    {0x3006, 0x40}, //GREEN
    {0x3007, 0x40}, //BLUE

    // RAW settings
    {0x3100,0x22},
    {0x3304,0x01},
    {0x3400,0x03},
    {0x3404,0x18},
    {0x3601, 0x00}  /* 10-bit mode */
};

static CamReg ov3640_vga[] ={
//const prog_char ov767_reg[CHANGE_REG_NUM+1][2]={
	    { 0x3012, 0x80},  /*Software reset the sensor */
	    { DELAY_MS,  0x05},    /* delay 5 milliseconds */

		{0x304d, 0x45},
		{0x30a7, 0x5e},
		{0x3087, 0x16},
		{0x309c, 0x1a},
		{0x30a2, 0xe4},
		{0x30aa, 0x42},
		{0x30b0, 0xff},
		{0x30b1, 0xff},
		{0x30b2, 0x10},
		{0x300e, 0x32},
		{0x300f, 0x21},
		{0x3010, 0x20},
	//	{0x3011, 0x4},
		{0x304c, 0x81},
		{0x30d7, 0x10},
		{0x30d9, 0xd},
		{0x30db, 0x8},
		{0x3016, 0x82},
		{0x3018, 0x38},
		{0x3019, 0x30},
		{0x301a, 0x61},
		{0x307d, 0x0},
		{0x3087, 0x2},
		{0x3082, 0x20},
		{0x3015, 0x12},
		{0x3014, 0x4},
		{0x3013, 0xf7},
		{0x303c, 0x8},
		{0x303d, 0x18},
		{0x303e, 0x6},
		{0x303f, 0xc},
		{0x3030, 0x62},
		{0x3031, 0x26},
		{0x3032, 0xe6},
		{0x3033, 0x6e},
		{0x3034, 0xea},
		{0x3035, 0xae},
		{0x3036, 0xa6},
		{0x3037, 0x6a},
		{0x3104, 0x2},
		{0x3105, 0xfd},
		{0x3106, 0x0},
		{0x3107, 0xff},
		{0x3300, 0x12},
		{0x3301, 0xde},
		{0x3302, 0xef},
		{0x3316, 0xff},
		{0x3317, 0x0},
		{0x3312, 0x26},
		{0x3314, 0x42},
		{0x3313, 0x2b},
		{0x3315, 0x42},
		{0x3310, 0xd0},
		{0x3311, 0xbd},
		{0x330c, 0x18},
		{0x330d, 0x18},
		{0x330e, 0x56},
		{0x330f, 0x5c},
		{0x330b, 0x1c},
		{0x3306, 0x5c},
		{0x3307, 0x11},
		{0x336a, 0x52},
		{0x3370, 0x46},
		{0x3376, 0x38},
		{0x3300, 0x13},
		{0x30b8, 0x20},
		{0x30b9, 0x17},
		{0x30ba, 0x4},
		{0x30bb, 0x8},
		{0x3507, 0x6},
		{0x350a, 0x4f},
		{0x3100, 0x2},
		{0x3301, 0xde},
		{0x3304, 0x0},
		{0x3400, 0x1},
		{0x3404, 0x11},
		{0x335f, 0x68},
		{0x3360, 0x18},
		{0x3361, 0xc},
		{0x3362, 0x12},
		{0x3363, 0x88},
		{0x3364, 0xe4},
		{0x3403, 0x42},
	//	{0x3088, 0x2},
	//	{0x3089, 0x80},
	//	{0x308a, 0x1},
	//	{0x308b, 0xe0},
		{0x308d, 0x4},
	//	{0x3086, 0x3},
		{0x3086, 0x0},
		{0x3011, 0x0},
		{0x304c, 0x85},
	//	{0x3600, 0xd0},
		{0x335f, 0x68},
		{0x3360, 0x18},
		{0x3361, 0xc},
		{0x3362, 0x1},
		{0x3363, 0x48},
		{0x3364, 0xf4},
		{0x3403, 0x42},

		{0x3088, H_PX >>8},// Hwidth
		{0x3089, H_PX & 0xff},//Hwidth 320
		{0x308a, V_PX >>8}, //VHeight
		{0x308b, V_PX & 0xff},//VHeight 240

		{0x307c, 0x12},
		{0x3090, 0xc8},
		{0x3600, 0xc4},

	{0x3362 ,(((V_PX+6) & 0xFF00)>>4) | (((H_PX+8) & 0xFF00)>>8)},
	{0x3363 ,((H_PX+8) & 0xFF)},
	{0x3364 ,((V_PX+6) & 0xFF)},
	{0x3403 ,0x42},
	{0x3088 ,(H_PX & 0xFF00)>>8},
	{0x3089 ,(H_PX & 0xFF)},
	{0x308a ,((V_PX+2)& 0xFF00)>>8},
	{0x308b ,((V_PX+2)& 0xFF)},
	{0x3f00 ,0x09},
	{0x3400 ,0x01},
	{0x3404 ,0x30},
		{0x307c, 0x12},
		{0x3090, 0xc8},
		{0x3600, 0xc4},

	{ARRAY_END, 0xff},
};



/* Sensor data from the CSI2 and the actual format are swapped.
 * ie to get UYVY format, we need to set the sensor to o/p
 * VYUY data format
 */

static CamReg OV3640_Format_YUV422_UYVYCommandsList[] = {
    { 0x3400, 0x00},
    { 0x3404, 0x00}, /* sensor o/p = YUV422 - YUYV */
    { 0x3600, 0xC4},
    {0xFF, 0xFF,   0x00}
};

static CamReg OV3640_Format_YUV422_YUY2CommandsList[] = {
    { 0x3400, 0x00},
    { 0x3404, 0x02}, /* sensor o/p = YUV422 - UYVY */
    { 0x3600, 0xC4},
    {0xFF, 0xFF,   0x00}
};

static CamReg OV3640_Format_RGB565_CommandsList[] = {
    { 0x3400, 0x00},
    { 0x3404, 0x30},
    { 0x3600, 0xC4},
    {0xFF, 0xFF,   0x00}
};

static CamReg OV3640_Format_RAW10_CommandsList[] = {
    { 0x3100, 0x22},
    { 0x3400, 0x00},
    { 0x3404, 0x18},
    { 0x3600, 0xC4},
    {0xFF, 0xFF,   0x00}
};

static CamReg OV3640_Format_RAW08_CommandsList[] = {
    /* Processed Raw */
    { 0x3400, 0x01},
    { 0x3404, 0x19},  //0x18 - org
    { 0x3600, 0xC4},
    {0xFF, 0xFF,   0x00}
};

/* The order of command list has been swapped because CSI2 receiver has a issue
 * in reading the sensor data. This issue is resolved by doing a swap in CCDC
 * so the result is the sensor and ccdc o/p are swapped.
 * If the pipeline is configured as YUY2,then we should configure UYVY for sensor
 * Similarly if the pipeline is configured for UYVY, then we should set YUY2 for sensor
 */
static CamReg* FormatSelect[] = {

    OV3640_Format_YUV422_YUY2CommandsList,
    OV3640_Format_YUV422_UYVYCommandsList,
    OV3640_Format_YUV422_UYVYCommandsList, //IJPG o/p  = sensor o/p raw(UYVY/YUY2/RGB/RAW)
    OV3640_Format_RGB565_CommandsList,
    OV3640_Format_RAW10_CommandsList,
    OV3640_Format_RAW08_CommandsList
};

static CamReg OV3640_PreviewXGA2QQVGA_CommandsList[] = {
    /* QQVGA:
	 * Width = 160 Height = 120
	 * Output Configuration: Width+8, Height+4
     */
    { 0x3302, 0xef},  /*  DSP_CRTL2 0->Reserved 0. */
    //{ 0x307c, 0x11},  //org value : 0x10, Flip included
    { 0x335f, 0x34},  /*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
    { 0x3360, 0x0c},  /*  hsize_in_L HsizeIn[7:0] */
    { 0x3361, 0x04},  /*  vsize_in_L VsizeIn[7:0] */
    { 0x3362, 0x00},  /*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
    { 0x3363, 0xA8},  /*  hsize_out_L HsizeOut[7;0] */
    { 0x3364, 0x7C},  /*  vsize_out_L VsizeOut[7:0] */
    { 0x3403, 0x42},  /*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
    { 0x3088, 0x00},  /*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
    { 0x3089, 0xA0},  /*  ISP_Xout[7:0] */
    { 0x308a, 0x00},  /*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
    { 0x308b, 0x78},  /*  ISP_Yout[7:0] */
	{0xFF, 0xFF,   0xFF}

};

static CamReg OV3640_PreviewQXGA2QQVGA_CommandsList[] = {

    /* Input: 2072x1548, Output: QQVGA:
	 * Width = 160 Height = 120
	 * Output Configuration: Width+8, Height+4
     */
    { 0x3302, 0xef},  /*  DSP_CRTL2 ->Reserved 0. */
    //{ 0x307c, 0x11},  //org value : 0x10, Flip included
    { 0x335f, 0x68},  /*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
    { 0x3360, 0x18},  /*  hsize_in_L HsizeIn[7:0] */
    { 0x3361, 0x0C},  /*  vsize_in_L VsizeIn[7:0] */
    { 0x3362, 0x00},  /*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
    { 0x3363, 0xA8},  /*  hsize_out_L HsizeOut[7;0] */
    { 0x3364, 0x7C},  /*  vsize_out_L VsizeOut[7:0] */
    { 0x3403, 0x42},  /*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
    { 0x3088, 0x00},  /*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
    { 0x3089, 0xA0},  /*  ISP_Xout[7:0] */
    { 0x308a, 0x00},  /*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
    { 0x308b, 0x78},  /*  ISP_Yout[7:0] */
	{0xFF, 0xFF,   0xFF}

};

static CamReg OV3640_PreviewXGA2QCIF_CommandsList[] = {
    /* QCIF:
	 * Width = 176 Height = 288
     */
    { 0x3302, 0xef},  /*  DSP_CRTL2 ->Reserved 0. */
    //{ 0x307c, 0x11},  //org value : 0x10, Flip included
    { 0x335f, 0x34},  /*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
    { 0x3360, 0x0c},  /*  hsize_in_L HsizeIn[7:0] */
    { 0x3361, 0x04},  /*  vsize_in_L VsizeIn[7:0] */
    { 0x3362, 0x00},  /*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
    { 0x3363, 0xb8},  /*  hsize_out_L HsizeOut[7;0] */
    { 0x3364, 0x94},  /*  vsize_out_L VsizeOut[7:0] */
    { 0x3403, 0x42},  /*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
    { 0x3088, 0x00},  /*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
    { 0x3089, 0xB0},  /*  ISP_Xout[7:0] */
    { 0x308a, 0x00},  /*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
    { 0x308b, 0x90},  /*  ISP_Yout[7:0] */
	{0xFF, 0xFF,   0xFF}

};

#if 1
static CamReg OV3640_PreviewXGA2QVGA_CommandsList[] = {
    /* QVGA:
	 * Width = 320 Height = 240
     */
    { 0x3302, 0xef},  /*  DSP_CRTL2 ->Reserved 0. */
    //{ 0x307c, 0x11},  //org value : 0x10, Flip included
    { 0x335f, 0x34},  /*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
    { 0x3360, 0x0c},  /*  hsize_in_L HsizeIn[7:0] */
    { 0x3361, 0x04},  /*  vsize_in_L VsizeIn[7:0] */
    { 0x3362, 0x01},  /*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
    { 0x3363, 0x48},  /*  hsize_out_L HsizeOut[7;0] */
    { 0x3364, 0xF4},  /*  vsize_out_L VsizeOut[7:0] */
    { 0x3403, 0x42},  /*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
    { 0x3088, 0x01},   /*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
    { 0x3089, 0x40},  /*  ISP_Xout[7:0] */
    { 0x308a, 0x00},  /*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
    { 0x308b, 0xF0},  /*  ISP_Yout[7:0] */
	{0xFF, 0xFF,   0xFF}

};
#else
static CamReg OV3640_PreviewXGA2QVGA_CommandsList[] = {
	/* QVGA:
	 * The default conf image from camera is H=320,V=240, But our display is H=240,V=320
	 * Options:
	 * a) display squashed image (by reversing H and V)
	 * b) rotate image by 90 degree and display
	 * c) maintain same ratio, but ask camera to display more H for V=320 (H=426.6666..)
	 * (a and b are confusing for production users, so I am going for c - in effect
	 * I will loose 1/3 of the horizontal image. (using H=420, V=320)
	 */
	{ 0x3302, 0xef},	/*  DSP_CRTL2 ->Reserved 0. */
	{ 0x335f, 0x34},	/*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
	{ 0x3360, 0x0c},	/*  hsize_in_L HsizeIn[7:0] */
	{ 0x3361, 0x04},	/*  vsize_in_L VsizeIn[7:0] */
	{ 0x3362, 0x11},	/*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
	{ 0x3363, 0xac},	/*  hsize_out_L HsizeOut[7;0] */
	{ 0x3364, 0x44},	/*  vsize_out_L VsizeOut[7:0] */
	{ 0x3403, 0x42},	/*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
	{ 0x3088, 0x11},	/*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
	{ 0x3089, 0xa4},	/*  ISP_Xout[7:0] */
	{ 0x308a, 0x11},	/*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
	{ 0x308b, 0x40},	/*  ISP_Yout[7:0] */
	{0xFF, 0xFF,   0x00}
};
#endif

static CamReg OV3640_PreviewXGA2VGA_CommandsList[] = {
	/* VGA:
	 * The default conf image from camera is H=640,V=480, But our display is H=480,V=640
	 * Options:
	 * a) display squashed image (by reversing H and V)
	 * b) rotate image by 90 degree and display
	 * c) maintain same ratio, but ask camera to display more H for V=640 (H=853.3333..)
	 * (a and b are confusing for production users, so I am going for c - in effect
	 * I will loose 1/3 of the horizontal image. (using H=850, V=640)
	 */
	{ 0x3302, 0xef},	/*  DSP_CRTL2 ->Reserved 0. */
	{ 0x335f, 0x34},	/*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
	{ 0x3360, 0x0c},	/*  hsize_in_L HsizeIn[7:0] */
	{ 0x3361, 0x04},	/*  vsize_in_L VsizeIn[7:0] */
	{ 0x3362, 0x12},	/*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
	{ 0x3363, 0x88},	/*  hsize_out_L HsizeOut[7;0] */
	{ 0x3364, 0xE4},	/*  vsize_out_L VsizeOut[7:0] */
	{ 0x3403, 0x42},	/*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
	{ 0x3088, 0x12},	/*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
	{ 0x3089, 0x80},	/*  ISP_Xout[7:0] */
	{ 0x308a, 0x11},	/*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
	{ 0x308b, 0xE0},	/*  ISP_Yout[7:0] */
	{0xFF, 0xFF,   0x00}
};

static CamReg OV3640_PreviewXGA2SIF_CommandsList[] = {
	/* QVGA */
	{ 0x3302, 0xef},	/*  DSP_CRTL2 ->Reserved 0. */
	{ 0x335f, 0x34},	/*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
	{ 0x3360, 0x0c},	/*  hsize_in_L HsizeIn[7:0] */
	{ 0x3361, 0x04},	/*  vsize_in_L VsizeIn[7:0] */
	{ 0x3362, 0x11},	/*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
	{ 0x3363, 0x68},	/*  hsize_out_L HsizeOut[7;0] */
	{ 0x3364, 0x50},	/*  vsize_out_L VsizeOut[7:0] */
	{ 0x3403, 0x42},	/*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
	{ 0x3088, 0x11},	/*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
	{ 0x3089, 0xa4},	/*  ISP_Xout[7:0] */
	{ 0x308a, 0x11},	/*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
	{ 0x308b, 0x40},	/*  ISP_Yout[7:0] */
	{0xFF, 0xFF,   0x00}
};


static CamReg OV3640_StillXGA2XGA_CommandsList[] = {

    { 0x3302, 0xef},	/*  DSP_CRTL2 ->Reserved 0. */
    { 0x335f, 0x34},	/*  Size in MISC ->Bit[3:0]=HsizeIn[11:8] [6:4]=VsizeIn[10:8] */
    { 0x3360, 0x0c},	/*  hsize_in_L HsizeIn[7:0] */
    { 0x3361, 0x04},	/*  vsize_in_L VsizeIn[7:0] */
    { 0x3362, 0x34},	/*  size_out_misc [3:0] = Hsize_out[11:8] [6:4] = Vszizeout[10:8] */
    { 0x3363, 0x0c},	/*  hsize_out_L HsizeOut[7;0] */
    { 0x3364, 0x04},	/*  vsize_out_L VsizeOut[7:0] */
    { 0x3403, 0x42},	/*  isp_pad_ctr2 [3:0] =YStart [7:4]=XStart */
    { 0x3088, 0x04},	/*  ISP_Xout[15:8] [3:0] = X_Size_out[11:8] */
    { 0x3089, 0x00},	/*  ISP_Xout[7:0] */
    { 0x308a, 0x03},	/*  ISP_YOut[15:8] [2:0] Y_size_out[10:8] */
    { 0x308b, 0x00},	/*  ISP_Yout[7:0] */
    {0xFF, 0xFF,   0x00}
};

static CamReg OV3640_dataPattern[] = {
	{ 0x307d, 0x80},	/* cbar */
	{ 0x3085, 0x20},
	{ 0x306c, 0x00},	/* blc off */
	{ 0x307b, 0x02},	/* cbar */
	{100,  0x0,    0x02},	/*  delay 2 milliseconds */
	{0xFF, 0xFF,   0x00}

};


static CamReg OV3640_PreviewCSI2_CommandsList[] = {
	{ 0x3507, 0x06},
	{ 0x350a, 0x4f},
	{ 0x3600, 0xc4},
	{ 0x3602, 0x20},	/* SOL off */
	{ 0x3104, 0x02},
	{ 0x3602, 0x22},	/* SOL/EOL off */
	{ 0x309e, 0x00},
	{ 0x3602, 0x22},	/* SOL/EOL off */
	{0xFF, 0xFF,   0x00}
};

static CamReg OV3640_StillOrientation[] = {
    { 0x307c, 0x11},  //Flip included
    {0xFF, 0xFF,   0x00}
};

static CamReg OV3640_PreviewOrientation[] = {
    { 0x307c, 0x10},  // no Flip
    {0xFF, 0xFF,   0x00}
};

static CamReg* PreviewResolution[] = {
    OV3640_PreviewXGA2QQVGA_CommandsList,
    //OV3640_PreviewQXGA2QQVGA_CommandsList,
    OV3640_PreviewXGA2QCIF_CommandsList,
    OV3640_PreviewXGA2QVGA_CommandsList,
    OV3640_PreviewXGA2VGA_CommandsList
};

static CamReg* StillResolution[] = {
    OV3640_PreviewXGA2QQVGA_CommandsList,
    OV3640_PreviewXGA2QCIF_CommandsList,
    OV3640_PreviewXGA2QVGA_CommandsList,
    OV3640_PreviewXGA2VGA_CommandsList,
    OV3640_StillXGA2XGA_CommandsList
};

#pragma pack()
const  xxov3640_reg[200][2]=
{
	{0x3012, 0x80},
	{DELAY_MS, 0x05},
	{0x304d, 0x45},
	{0x30a7, 0x5e},
	{0x3087, 0x16},
	{0x309c, 0x1a},
	{0x30a2, 0xe4},
	{0x30aa, 0x42},
	{0x30b0, 0xff},
	{0x30b1, 0xff},
	{0x30b2, 0x10},
	{0x300e, 0x32},
	{0x300f, 0x21},
	{0x3010, 0x20},
//	{0x3011, 0x4},
	{0x304c, 0x81},
	{0x30d7, 0x10},
	{0x30d9, 0xd},
	{0x30db, 0x8},
	{0x3016, 0x82},
	{0x3018, 0x38},
	{0x3019, 0x30},
	{0x301a, 0x61},
	{0x307d, 0x0},
	{0x3087, 0x2},
	{0x3082, 0x20},
	{0x3015, 0x12},
	{0x3014, 0x4},
	{0x3013, 0xf7},
	{0x303c, 0x8},
	{0x303d, 0x18},
	{0x303e, 0x6},
	{0x303f, 0xc},
	{0x3030, 0x62},
	{0x3031, 0x26},
	{0x3032, 0xe6},
	{0x3033, 0x6e},
	{0x3034, 0xea},
	{0x3035, 0xae},
	{0x3036, 0xa6},
	{0x3037, 0x6a},
	{0x3104, 0x2},
	{0x3105, 0xfd},
	{0x3106, 0x0},
	{0x3107, 0xff},
	{0x3300, 0x12},
	{0x3301, 0xde},
	{0x3302, 0xef},
	{0x3316, 0xff},
	{0x3317, 0x0},
	{0x3312, 0x26},
	{0x3314, 0x42},
	{0x3313, 0x2b},
	{0x3315, 0x42},
	{0x3310, 0xd0},
	{0x3311, 0xbd},
	{0x330c, 0x18},
	{0x330d, 0x18},
	{0x330e, 0x56},
	{0x330f, 0x5c},
	{0x330b, 0x1c},
	{0x3306, 0x5c},
	{0x3307, 0x11},
	{0x336a, 0x52},
	{0x3370, 0x46},
	{0x3376, 0x38},
	{0x3300, 0x13},
	{0x30b8, 0x20},
	{0x30b9, 0x17},
	{0x30ba, 0x4},
	{0x30bb, 0x8},
	{0x3507, 0x6},
	{0x350a, 0x4f},
	{0x3100, 0x2},
	{0x3301, 0xde},
	{0x3304, 0x0},
	{0x3400, 0x1},
	{0x3404, 0x11},
	{0x335f, 0x68},
	{0x3360, 0x18},
	{0x3361, 0xc},
	{0x3362, 0x12},
	{0x3363, 0x88},
	{0x3364, 0xe4},
	{0x3403, 0x42},
//	{0x3088, 0x2},
//	{0x3089, 0x80},
//	{0x308a, 0x1},
//	{0x308b, 0xe0},
	{0x308d, 0x4},
//	{0x3086, 0x3},
	{0x3086, 0x0},
	{0x3011, 0x0},
	{0x304c, 0x85},
//	{0x3600, 0xd0},
	{0x335f, 0x68},
	{0x3360, 0x18},
	{0x3361, 0xc},
	{0x3362, 0x1},
	{0x3363, 0x48},
	{0x3364, 0xf4},
	{0x3403, 0x42},

	{0x3088, H_PX >>8},// Hwidth
	{0x3089, H_PX & 0xff},//Hwidth 320
	{0x308a, V_PX >>8}, //VHeight
	{0x308b, V_PX & 0xff},//VHeight 240

	{0x307c, 0x12},
	{0x3090, 0xc8},
	{0x3600, 0xc4},

{0x3362 ,(((V_PX+6) & 0xFF00)>>4) | (((H_PX+8) & 0xFF00)>>8)},
{0x3363 ,((H_PX+8) & 0xFF)},
{0x3364 ,((V_PX+6) & 0xFF)},
{0x3403 ,0x42},
{0x3088 ,(H_PX & 0xFF00)>>8},
{0x3089 ,(H_PX & 0xFF)},
{0x308a ,((V_PX+2)& 0xFF00)>>8},
{0x308b ,((V_PX+2)& 0xFF)},
{0x3f00 ,0x09},
{0x3400 ,0x01},
{0x3404 ,0x30},
	{0x307c, 0x12},
	{0x3090, 0xc8},
	{0x3600, 0xc4},

	{ARRAY_END, 0xff},
};


#endif // __OV3640_H_INCLUDED__
//kk


